from maggma.api.resource import SubmissionResource
from emmet.api.routes._consumer.query_operator import (
    UserSettingsPostQuery,
    UserSettingsGetQuery,
)
from emmet.core._user_settings import UserSettingsDoc


def settings_resource(consumer_settings_store):
    resource = SubmissionResource(
        consumer_settings_store,
        UserSettingsDoc,
        post_query_operators=[UserSettingsPostQuery()],
        get_query_operators=[UserSettingsGetQuery()],
        enable_default_search=False,
        include_in_schema=False,
    )

    return resource



from maggma.api.resource import SubmissionResource
from emmet.api.routes._general_store.query_operator import (
    GeneralStorePostQuery,
    GeneralStoreGetQuery,
)
from maggma.api.query_operator import PaginationQuery, SparseFieldsQuery
from emmet.core._general_store import GeneralStoreDoc


def general_store_resource(general_store):
    resource = SubmissionResource(
        general_store,
        GeneralStoreDoc,
        post_query_operators=[GeneralStorePostQuery()],
        get_query_operators=[
            GeneralStoreGetQuery(),
            PaginationQuery(),
            SparseFieldsQuery(
                model=GeneralStoreDoc,
                default_fields=["kind", "markdown", "meta", "last_updated"],
            ),
        ],
        enable_default_search=True,
        include_in_schema=False,
        calculate_submission_id=True,
    )

    return resource



from maggma.api.query_operator import PaginationQuery, SparseFieldsQuery
from maggma.api.resource import ReadOnlyResource
from emmet.api.core.global_header import GlobalHeaderProcessor
from emmet.api.core.settings import MAPISettings

from emmet.core.similarity import SimilarityDoc


def similarity_resource(similarity_store):
    resource = ReadOnlyResource(
        similarity_store,
        SimilarityDoc,
        query_operators=[
            PaginationQuery(),
            SparseFieldsQuery(SimilarityDoc, default_fields=["material_id"]),
        ],
        header_processor=GlobalHeaderProcessor(),
        tags=["Similarity"],
        enable_default_search=False,
        disable_validation=True,
        timeout=MAPISettings().TIMEOUT
    )

    return resource



from fastapi import Query, Body
from pymatgen.core.structure import Structure
from maggma.api.utils import STORE_PARAMS
from maggma.api.query_operator import QueryOperator


class MPCompletePostQuery(QueryOperator):
    """Query operators to provide MPComplete data to post"""

    def query(
        self,
        structure: Structure = Body(..., title="Structure submission"),
        public_name: str = Query(..., title="Public name"),
        public_email: str = Query(..., title="Public email"),
    ) -> STORE_PARAMS:

        self.structure = structure
        self.public_name = public_name
        self.public_email = public_email

        crit = {
            "structure": structure,
            "public_email": public_email,
            "public_name": public_name,
        }

        return {"criteria": crit}

    def post_process(self, docs, query):

        d = [
            {
                "structure": query["criteria"]["structure"],
                "public_email": query["criteria"]["public_email"],
                "public_name": query["criteria"]["public_name"],
            }
        ]

        return d


class MPCompleteGetQuery(QueryOperator):
    """Query operators for querying on MPComplete data"""

    def query(
        self,
        public_name: str = Query(None, title="Public name"),
        public_email: str = Query(None, title="Public email"),
    ) -> STORE_PARAMS:

        self.public_name = public_name
        self.public_email = public_email

        crit = {}

        if public_name is not None:
            crit.update({"public_name": public_name})

        if public_email is not None:
            crit.update({"public_email": public_email})

        return {"criteria": crit}



from maggma.api.resource import ReadOnlyResource
from maggma.api.resource.aggregation import AggregationResource
from maggma.api.query_operator import PaginationQuery, SparseFieldsQuery

from emmet.api.routes.robocrys.query_operators import RoboTextSearchQuery
from emmet.api.routes.materials.query_operators import MultiMaterialIDQuery
from emmet.core.robocrys import RobocrystallogapherDoc
from emmet.api.core.global_header import GlobalHeaderProcessor
from emmet.api.core.settings import MAPISettings

timeout = MAPISettings().TIMEOUT


def robo_resource(robo_store):
    resource = ReadOnlyResource(
        robo_store,
        RobocrystallogapherDoc,
        query_operators=[
            MultiMaterialIDQuery(),
            PaginationQuery(),
            SparseFieldsQuery(
                RobocrystallogapherDoc, default_fields=["material_id", "last_updated"]
            ),
        ],
        header_processor=GlobalHeaderProcessor(),
        tags=["Robocrystallographer"],
        disable_validation=True,
        timeout=timeout,
    )

    return resource


def robo_search_resource(robo_store):
    resource = AggregationResource(
        robo_store,
        RobocrystallogapherDoc,
        pipeline_query_operator=RoboTextSearchQuery(),
        tags=["Robocrystallographer"],
        sub_path="/text_search/",
        header_processor=GlobalHeaderProcessor(),
        timeout=timeout,
    )

    return resource



from maggma.api.resource import HintScheme


class SummaryHintScheme(HintScheme):
    """
    Hint scheme for the summary endpoint.
    """

    def generate_hints(self, query):

        if "nelements" in query["criteria"]:
            return {"hint": {"nelements": 1}}
        elif "has_props" in query["criteria"]:
            return {"hint": {"has_props": 1}}
        else:
            return {"hint": {}}



"""
This script converts synthesis recipes data fetched directly
from Ceder Group Synthesis Mining team MongoDB into MP compatible
formats.
"""
import json
import os
import re
import logging

from pymatgen.core.composition import CompositionError, Composition
from pymongo import MongoClient

logger = logging.getLogger(__name__)


def convert_value(val):
    """Convert values in operation conditions dictionaries."""
    return {
        "min_value": float(val["min"]) if val["min"] is not None else None,
        "max_value": float(val["max"]) if val["max"] is not None else None,
        "values": [float(x) for x in val["values"]],
        "units": str(val["units"]),
    }


def convert_conditions(cond, op_type):
    """Convert conditions dictionaries."""
    return {
        "heating_temperature": [convert_value(x) for x in cond["temperature"]],
        "heating_time": [convert_value(x) for x in cond["time"]],
        "heating_atmosphere": [x.strip() for x in cond["environment"] if x.strip()]
        if op_type == "HeatingOperation"
        else [],
        "mixing_device": (
            cond["environment"][1].strip() if cond["environment"][1].strip() else None
        )
        if op_type == "MixingOperation"
        else None,
        "mixing_media": (
            cond["environment"][0].strip() if cond["environment"][0].strip() else None
        )
        if op_type == "MixingOperation"
        else None,
    }


all_posible_ops = set()


def convert_op(op):
    """Convert operation dictionaries."""
    all_posible_ops.add(op["type"])
    return {
        "type": op["type"],
        "token": op["string"],
        "conditions": convert_conditions(op["attributes"], op["type"]),
    }


def convert_mat_value(val):
    """Convert values specified in materials elements_vars."""
    return {
        "values": [float(x) for x in val["values"]],
        "min_value": float(val["min_value"]) if val["min_value"] is not None else None,
        "max_value": float(val["max_value"]) if val["max_value"] is not None else None,
    }


def convert_material(mat):
    """Convert materials dictionaries."""
    return {
        "material_string": str(mat["material_string"]),
        "material_name": str(mat["material_name"]),
        "material_formula": str(mat["material_formula"]),
        "phase": str(mat["phase"]) or None,
        "is_acronym": bool(mat["is_acronym"]),
        "composition": [
            {
                "formula": str(x["formula"]),
                "amount": str(x["amount"]),
                "elements": {str(y): str(z) for y, z in x["elements"].items()},
            }
            for x in mat["composition"]
        ],
        "amounts_vars": {
            x: convert_mat_value(y) for x, y in mat["amounts_vars"].items()
        },
        "elements_vars": {
            x: [str(z.strip()) for z in y if z.strip()]
            for x, y in mat["elements_vars"].items()
        },
        "additives": [str(x.strip()) for x in mat["additives"] if x.strip()],
        "oxygen_deficiency": str(mat["oxygen_deficiency"])
        if mat["oxygen_deficiency"]
        else None,
    }


def get_material_formula(mat):
    """Convert string material formulas into pymatgen Compositions."""
    formula = mat["material_formula"]
    formula = re.sub(r"·\d*H2O", "", formula)
    try:
        return Composition(formula)
    except (CompositionError, ValueError):
        q = None
        for comp in mat["composition"]:
            if q is None:
                q = Composition(
                    {x: float(y) for x, y in comp["elements"].items()}
                ) * float(comp["amount"])
            else:
                q += Composition(
                    {x: float(y) for x, y in comp["elements"].items()}
                ) * float(comp["amount"])
        return q


def target_comps(doc):
    """Find all target material formulas and convert them into Composition."""
    result = []
    for x in doc["targets_string"]:
        if not x.strip():
            continue
        try:
            result.append(Composition(x))
        except (CompositionError, ValueError):
            pass
    return result


def precursor_comps(doc):
    """Find all precursor material formulas and convert them into Composition."""
    result = []
    for x in doc["precursors"]:
        try:
            result.append(get_material_formula(x))
        except (CompositionError, ValueError):
            pass
    return result


def convert_one(doc):
    """Convert an entire synthesis recipe."""
    return {
        "doi": str(doc["doi"]),
        "paragraph_string": " ".join(doc["ext_paragraph"]),
        "synthesis_type": str(doc["synthesis_type"]),
        "reaction_string": str(doc["reaction_string"]),
        "reaction": {
            "left_side": [
                {"amount": str(x["amount"]), "material": str(x["material"])}
                for x in doc["reaction"]["left"]
            ],
            "right_side": [
                {"amount": str(x["amount"]), "material": str(x["material"])}
                for x in doc["reaction"]["right"]
            ],
        },
        "targets_formula": [x.formula for x in target_comps(doc)],
        "target": convert_material(doc["target"]),
        "targets_formula_s": [x.reduced_formula for x in target_comps(doc)],
        "precursors_formula_s": [x.reduced_formula for x in precursor_comps(doc)],
        "precursors_formula": [x.formula for x in precursor_comps(doc)],
        "precursors": [convert_material(x) for x in doc["precursors"]],
        "operations": [convert_op(x) for x in doc.get("operations", [])],
    }


def main():
    """
    Convert the Reactions_Solid_State/Reactions_Sol_Gel collection in
    Ceder Group database into a json file which can be imported into the MP database.
    """
    synpro_db = MongoClient(os.environ["SYNPRO_URI"]).SynPro

    synthesis_recipes = []

    for item in synpro_db.Reactions_Solid_State.find():
        synthesis_recipes.append(convert_one(item))
    for item in synpro_db.Reactions_Sol_Gel.find():
        synthesis_recipes.append(convert_one(item))

    with open("synthesis_recipes.json", "w") as f:
        json.dump(synthesis_recipes, f)

    logger.info(f"All possible operation types {all_posible_ops}")


if __name__ == "__main__":
    main()



from emmet.core.mpid import MPID

from maggma.api.query_operator import QueryOperator
from maggma.api.utils import STORE_PARAMS

from fastapi import Path


class PhononImgQuery(QueryOperator):
    """
    Method to generate a query on phonon image data.
    """

    def query(
        self,
        task_id: MPID = Path(
            ...,
            description="The calculation (task) ID associated with the data object",
        ),
    ) -> STORE_PARAMS:

        return {"criteria": {"task_id": str(task_id)}}



from maggma.api.query_operator.dynamic import NumericQuery
from maggma.api.resource import ReadOnlyResource
from emmet.core.electrode import InsertionElectrodeDoc
from emmet.api.core.global_header import GlobalHeaderProcessor

from maggma.api.query_operator import PaginationQuery, SortQuery, SparseFieldsQuery
from emmet.api.routes.electrodes.query_operators import (
    ElectrodeFormulaQuery,
    ElectrodeElementsQuery,
    ElectrodesChemsysQuery,
    WorkingIonQuery,
    ElectrodeMultiMaterialIDQuery,
    MultiBatteryIDQuery
)

from emmet.api.core.settings import MAPISettings


def insertion_electrodes_resource(insertion_electrodes_store):
    resource = ReadOnlyResource(
        insertion_electrodes_store,
        InsertionElectrodeDoc,
        query_operators=[
            MultiBatteryIDQuery(),
            ElectrodeMultiMaterialIDQuery(),
            ElectrodeFormulaQuery(),
            ElectrodesChemsysQuery(),
            WorkingIonQuery(),
            ElectrodeElementsQuery(),
            NumericQuery(model=InsertionElectrodeDoc),
            SortQuery(),
            PaginationQuery(),
            SparseFieldsQuery(
                InsertionElectrodeDoc, default_fields=["battery_id", "last_updated"],
            ),
        ],
        header_processor=GlobalHeaderProcessor(),
        tags=["Electrodes"],
        disable_validation=True,
        timeout=MAPISettings().TIMEOUT
    )

    return resource



from typing import Optional
from fastapi import Query
from maggma.api.query_operator import QueryOperator
from maggma.api.utils import STORE_PARAMS

from collections import defaultdict


class DielectricQuery(QueryOperator):
    """
    Method to generate a query for ranges of dielectric constant data
    """

    def query(
        self,
        e_total_max: Optional[float] = Query(
            None, description="Maximum value for the total dielectric constant.",
        ),
        e_total_min: Optional[float] = Query(
            None, description="Minimum value for the total dielectric constant.",
        ),
        e_ionic_max: Optional[float] = Query(
            None, description="Maximum value for the ionic dielectric constant.",
        ),
        e_ionic_min: Optional[float] = Query(
            None, description="Minimum value for the ionic dielectric constant.",
        ),
        e_electronic_max: Optional[float] = Query(
            None, description="Maximum value for the electronic dielectric constant.",
        ),
        e_electronic_min: Optional[float] = Query(
            None, description="Minimum value for the electronic dielectric constant.",
        ),
        n_max: Optional[float] = Query(
            None, description="Maximum value for the refractive index.",
        ),
        n_min: Optional[float] = Query(
            None, description="Minimum value for the refractive index.",
        ),
    ) -> STORE_PARAMS:

        crit = defaultdict(dict)  # type: dict

        d = {
            "e_total": [e_total_min, e_total_max],
            "e_ionic": [e_ionic_min, e_ionic_max],
            "e_electronic": [e_electronic_min, e_electronic_max],
            "n": [n_min, n_max],
        }

        for entry in d:
            if d[entry][0] is not None:
                crit[entry]["$gte"] = d[entry][0]

            if d[entry][1] is not None:
                crit[entry]["$lte"] = d[entry][1]

        return {"criteria": crit}

    def ensure_indexes(self):  # pragma: no cover
        keys = [
            "e_total",
            "e_ionic",
            "e_electronic",
            "n",
        ]
        return [(key, False) for key in keys]



from typing import Optional
from fastapi import Query
from maggma.api.query_operator import QueryOperator
from maggma.api.utils import STORE_PARAMS

from collections import defaultdict


class PiezoelectricQuery(QueryOperator):
    """
    Method to generate a query for ranges of piezoelectric data
    """

    def query(
        self,
        piezo_modulus_max: Optional[float] = Query(
            None, description="Maximum value for the piezoelectric modulus in C/m².",
        ),
        piezo_modulus_min: Optional[float] = Query(
            None, description="Minimum value for the piezoelectric modulus in C/m².",
        ),
    ) -> STORE_PARAMS:

        crit = defaultdict(dict)  # type: dict

        d = {
            "e_ij_max": [piezo_modulus_min, piezo_modulus_max],
        }

        for entry in d:
            if d[entry][0] is not None:
                crit[entry]["$gte"] = d[entry][0]

            if d[entry][1] is not None:
                crit[entry]["$lte"] = d[entry][1]

        return {"criteria": crit}

    def ensure_indexes(self):  # pragma: no cover
        return [("e_ij_max", False)]



from maggma.api.resource import AggregationResource
from emmet.api.core.settings import MAPISettings
from emmet.api.routes.synthesis.query_operators import SynthesisSearchQuery
from emmet.api.core.global_header import GlobalHeaderProcessor
from emmet.core.synthesis.core import SynthesisSearchResultModel

synth_indexes = [
    ("synthesis_type", False),
    ("targets_formula_s", False),
    ("precursors_formula_s", False),
    ("operations.type", False),
    ("operations.conditions.heating_temperature.values", False),
    ("operations.conditions.heating_time.values", False),
    ("operations.conditions.heating_atmosphere", False),
    ("operations.conditions.mixing_device", False),
    ("operations.conditions.mixing_media", False),
]


def synth_resource(synth_store):

    resource = AggregationResource(
        synth_store,
        SynthesisSearchResultModel,
        tags=["Synthesis"],
        pipeline_query_operator=SynthesisSearchQuery(),
        header_processor=GlobalHeaderProcessor(),
        timeout=MAPISettings().TIMEOUT
    )

    return resource



from maggma.api.resource import ReadOnlyResource
from emmet.core.polar import DielectricDoc

from maggma.api.query_operator import PaginationQuery, SortQuery, SparseFieldsQuery
from emmet.api.routes.dielectric.query_operators import DielectricQuery
from emmet.api.routes.materials.query_operators import MultiMaterialIDQuery
from emmet.api.core.global_header import GlobalHeaderProcessor
from emmet.api.core.settings import MAPISettings


def dielectric_resource(dielectric_store):
    resource = ReadOnlyResource(
        dielectric_store,
        DielectricDoc,
        query_operators=[
            MultiMaterialIDQuery(),
            DielectricQuery(),
            SortQuery(),
            PaginationQuery(),
            SparseFieldsQuery(DielectricDoc, default_fields=["material_id", "last_updated"]),
        ],
        header_processor=GlobalHeaderProcessor(),
        tags=["Dielectric"],
        disable_validation=True,
        timeout=MAPISettings().TIMEOUT,
    )

    return resource



from fastapi import Query
from maggma.api.query_operator import QueryOperator
from maggma.api.utils import STORE_PARAMS
from emmet.core.xas import Edge, Type
from pymatgen.core.periodic_table import Element
from typing import Optional


class XASQuery(QueryOperator):
    def query(
        self,
        edge: Edge = Query(None, title="XAS Edge"),
        spectrum_type: Type = Query(None, title="Spectrum Type"),
        absorbing_element: Element = Query(None, title="Absorbing Element"),
    ) -> STORE_PARAMS:
        """
        Query parameters unique to XAS
        """
        query = {
            "edge": edge.value if edge else None,
            "absorbing_element": str(absorbing_element) if absorbing_element else None,
            "spectrum_type": str(spectrum_type.value) if spectrum_type else None,
        }
        query = {k: v for k, v in query.items() if v}

        return {"criteria": query} if len(query) > 0 else {}

    def ensure_indexes(self):  # pragma: no cover
        keys = ["edge", "absorbing_element", "spectrum_type"]
        return [(key, False) for key in keys]


class XASTaskIDQuery(QueryOperator):
    """
    Method to generate a query for XAS data given a list of task_ids
    """

    def query(
        self,
        material_ids: Optional[str] = Query(
            None, description="Comma-separated list of material_id to query on"
        ),
    ) -> STORE_PARAMS:

        crit = {}

        if material_ids:
            crit.update(
                {
                    "material_id": {
                        "$in": [
                            material_id.strip()
                            for material_id in material_ids.split(",")
                        ]
                    }
                }
            )

        return {"criteria": crit}



from maggma.api.query_operator.dynamic import NumericQuery
from maggma.api.resource import ReadOnlyResource
from emmet.core.eos import EOSDoc

from maggma.api.query_operator import PaginationQuery, SortQuery, SparseFieldsQuery
from emmet.api.core.global_header import GlobalHeaderProcessor
from emmet.api.core.settings import MAPISettings


def eos_resource(eos_store):
    resource = ReadOnlyResource(
        eos_store,
        EOSDoc,
        query_operators=[
            NumericQuery(model=EOSDoc),
            SortQuery(),
            PaginationQuery(),
            SparseFieldsQuery(EOSDoc, default_fields=["task_id"]),
        ],
        header_processor=GlobalHeaderProcessor(),
        tags=["EOS"],
        disable_validation=True,
        timeout=MAPISettings().TIMEOUT
    )

    return resource
